/*
 * Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
 *
 * This file is part of NRAO Workspaces.
 *
 * Workspaces is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Workspaces is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
 */
import { Component, EventEmitter, Input, OnInit, Output } from "@angular/core";
import { CapabilityRequest } from "../../../../model/capability-request";
import { Capability } from "../../../../model/capability";
import { CapabilityLauncherService } from "../../../../services/capability-launcher.service";
import { WorkflowLauncherService } from "../../../../services/workflow-launcher.service";
import { CapabilityExecution } from "../../../../model/capability-execution";
import { CapabilityRequestService } from "../../../../services/capability-request.service";
import { CapabilityVersion } from "../../../../model/capability-version";

@Component({
  selector: "app-request-operations",
  templateUrl: "./request-operations.component.html",
  styleUrls: ["./request-operations.component.scss"],
})
export class RequestOperationsComponent implements OnInit {
  constructor(
    public capabilityLauncherService: CapabilityLauncherService,
    public workflowLauncherService: WorkflowLauncherService,
    private capabilityRequestService: CapabilityRequestService,
  ) {}
  @Input() capabilityRequest: CapabilityRequest;
  @Input() latestVersion: CapabilityVersion;
  @Input() selectedVersion: CapabilityVersion;
  @Input() capability: Capability;
  public capabilityExecution: CapabilityExecution;
  @Output() cartaLaunched: EventEmitter<any> = new EventEmitter<any>();
  public cartaResponse: any;
  public hasBeenClicked: boolean = false;
  public userEmail: string;

  // Observer for submitting capability request objects (returns a capability execution)
  public submitRequestObserver = {
    next: (submitResponse: CapabilityExecution) => {
      this.capabilityExecution = submitResponse;
      window.location.reload();
    },
    error: (error) => {
      console.log(error);
    },
  };

  public launchCartaObserver = {
    next: (resp: any) => {
      this.cartaResponse = resp;
      this.hasBeenClicked = true;
      this.cartaLaunched.emit({ type: "carta", email: this.userEmail });
    },
    error: (error) => {
      console.log(error);

      this.cartaLaunched.emit({ type: "error" });
    },
  };

  public followonRequestObserver = {
    next: (followonResponse: CapabilityRequest) => {
      if (followonResponse.id) {
        // Capability request created; ID found
        // Redirect to request status page
        this.capabilityRequestService.redirectToRequestStatusPage(followonResponse.id);
      }
    },
    error: (error) => {
      console.log(error);
    },
  };

  launchCarta($event) {
    let params = {
      need_data: false,
      notify_ready: true,
      product_locator: null,
      user_email: this.userEmail,
      parent_wf_request_id: this.selectedVersion.current_execution.current_workflow_request_id
    };
    this.workflowLauncherService
      .createAndSubmitWorkflow("carta", params)
      .subscribe(this.launchCartaObserver);
  }

  ngOnInit(): void {
    if (this.capabilityRequest.parameters.user_email !== undefined) {
      this.userEmail = this.capabilityRequest.parameters.user_email.toString();
    } else {
      this.userEmail = null;
    }
  }
}
