#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.

# pylint: disable=C0114, C0116

from abc import ABC
from typing import List

from workspaces.capability.schema import CapabilityRequest, CapabilityVersion
from workspaces.notification.schema import Base, NotificationTemplate


class NotificationInfoIF(ABC):
    """
    Data access object that can look up notification templates etc.
    """

    def all_templates(self) -> List[NotificationTemplate]:
        raise NotImplementedError

    def lookup_template(self, template_name: str) -> NotificationTemplate:
        raise NotImplementedError

    def create_template(self, template_name: str, description: str, content: str) -> NotificationTemplate:
        raise NotImplementedError

    def delete_template(self, template_name: str):
        raise NotImplementedError

    def save_entity(self, entity: Base):
        pass


class NotificationServiceIF(ABC):
    """
    The notification service: clients access this to send user notification emails
    """

    def notify_submitted(self, version: CapabilityVersion):
        raise NotImplementedError

    def notify_complete(self, version: CapabilityVersion):
        raise NotImplementedError

    def notify_failed(self, version: CapabilityVersion):
        raise NotImplementedError

    def notify_qa_ready(self, request: CapabilityRequest):
        pass

    def notify_ingestion_complete(self, version: CapabilityVersion):
        pass

    def notify_ingestion_failed(self, capability_request):
        pass
